@@ -13,9 +13,14 @@ const {
   prepareMainThreadExecution,
   markBootstrapComplete,
 } = require('internal/process/pre_execution');
-const { evalModuleEntryPoint, evalScript } = require('internal/process/execution');
+const {
+  evalModuleEntryPoint,
+  evalTypeScript,
+  parseAndEvalCommonjsTypeScript,
+  parseAndEvalModuleTypeScript,
+  evalScript,
+} = require('internal/process/execution');
 const { addBuiltinLibsToObject } = require('internal/modules/helpers');
-const { stripTypeScriptModuleTypes } = require('internal/modules/typescript');
 const { getOptionValue } = require('internal/options');
 
 prepareMainThreadExecution();
@@ -23,18 +28,19 @@ addBuiltinLibsToObject(globalThis, '<eval>');
 markBootstrapComplete();
 
 const code = getOptionValue('--eval');
-const source = getOptionValue('--experimental-strip-types') ?
-  stripTypeScriptModuleTypes(code) :
-  code;
 
 const print = getOptionValue('--print');
 const shouldLoadESM = getOptionValue('--import').length > 0 || getOptionValue('--experimental-loader').length > 0;
-if (getOptionValue('--input-type') === 'module') {
-  evalModuleEntryPoint(source, print);
+const inputType = getOptionValue('--input-type');
+const tsEnabled = getOptionValue('--strip-types');
+if (inputType === 'module') {
+  evalModuleEntryPoint(code, print);
+} else if (inputType === 'module-typescript' && tsEnabled) {
+  parseAndEvalModuleTypeScript(code, print);
 } else {
   // For backward compatibility, we want the identifier crypto to be the
   // `node:crypto` module rather than WebCrypto.
-  const isUsingCryptoIdentifier = RegExpPrototypeExec(/\bcrypto\b/, source) !== null;
+  const isUsingCryptoIdentifier = RegExpPrototypeExec(/\bcrypto\b/, code) !== null;
   const shouldDefineCrypto = isUsingCryptoIdentifier && internalBinding('config').hasOpenSSL;
 
   if (isUsingCryptoIdentifier && !shouldDefineCrypto) {
@@ -49,11 +55,24 @@ if (getOptionValue('--input-type') === 'module') {
     };
     ObjectDefineProperty(object, name, { __proto__: null, set: setReal });
   }
-  evalScript('[eval]',
-             shouldDefineCrypto ? (
-               print ? `let crypto=require("node:crypto");{${source}}` : `(crypto=>{{${source}}})(require('node:crypto'))`
-             ) : source,
-             getOptionValue('--inspect-brk'),
-             print,
-             shouldLoadESM);
+
+  let evalFunction;
+  if (inputType === 'commonjs') {
+    evalFunction = evalScript;
+  } else if (inputType === 'commonjs-typescript' && tsEnabled) {
+    evalFunction = parseAndEvalCommonjsTypeScript;
+  } else if (tsEnabled) {
+    evalFunction = evalTypeScript;
+  } else {
+    // Default to commonjs.
+    evalFunction = evalScript;
+  }
+
+  evalFunction('[eval]',
+               shouldDefineCrypto ? (
+                 print ? `let crypto=require("node:crypto");{${code}}` : `(crypto=>{{${code}}})(require('node:crypto'))`
+               ) : code,
+               getOptionValue('--inspect-brk'),
+               print,
+               shouldLoadESM);
 }
